use actix_web::{web, HttpResponse, Responder};
use rusqlite::{params, Connection};
use std::sync::{Arc, Mutex};
use uuid::Uuid;

use crate::models::{Form, Submission};

// Create a new form
pub async fn create_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    form: web::Json<Form>,
) -> impl Responder {
    let conn = db.lock().unwrap(); // Lock the Mutex to access the database
    let form_id = Uuid::new_v4().to_string();
    let form_json = serde_json::to_string(&form.fields).unwrap();

    match conn.execute(
        "INSERT INTO forms (id, name, fields) VALUES (?1, ?2, ?3)",
        params![form_id, form.name, form_json],
    ) {
        Ok(_) => HttpResponse::Ok().json(form_id),
        Err(e) => HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    }
}

// Get all forms
pub async fn get_forms(db: web::Data<Arc<Mutex<Connection>>>) -> impl Responder {
    let conn = db.lock().unwrap(); // Lock the Mutex to access the database

    let mut stmt = match conn.prepare("SELECT id, name, fields FROM forms") {
        Ok(stmt) => stmt,
        Err(e) => return HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    };

    let forms_iter = stmt
        .query_map([], |row| {
            let id: String = row.get(0)?;
            let name: String = row.get(1)?;
            let fields: String = row.get(2)?;
            let fields = serde_json::from_str(&fields).unwrap();
            Ok(Form { id, name, fields })
        })
        .unwrap();

    let forms: Vec<Form> = forms_iter.filter_map(|f| f.ok()).collect();
    HttpResponse::Ok().json(forms)
}

// Submit a form
pub async fn submit_form(
    db: web::Data<Arc<Mutex<Connection>>>,
    path: web::Path<String>,
    submission: web::Json<serde_json::Value>,
) -> impl Responder {
    let conn = db.lock().unwrap(); // Lock the Mutex to access the database
    let submission_id = Uuid::new_v4().to_string();
    let form_id = path.into_inner();
    let submission_json = serde_json::to_string(&submission.into_inner()).unwrap();

    match conn.execute(
        "INSERT INTO submissions (id, form_id, data) VALUES (?1, ?2, ?3)",
        params![submission_id, form_id, submission_json],
    ) {
        Ok(_) => HttpResponse::Ok().json(submission_id),
        Err(e) => HttpResponse::InternalServerError().body(format!("Error: {}", e)),
    }
}

// Get submissions for a form
pub async fn get_submissions(
    db: web::Data<Arc<Mutex<Connection>>>,
    path: web::Path<String>,
) -> impl Responder {
    let conn = db.lock().unwrap(); // Lock the Mutex to access the database
    let form_id = path.into_inner();

    let mut stmt =
        match conn.prepare("SELECT id, form_id, data FROM submissions WHERE form_id = ?1") {
            Ok(stmt) => stmt,
            Err(e) => return HttpResponse::InternalServerError().body(format!("Error: {}", e)),
        };

    let submissions_iter = stmt
        .query_map([form_id], |row| {
            let id: String = row.get(0)?;
            let form_id: String = row.get(1)?;
            let data: String = row.get(2)?;
            let data = serde_json::from_str(&data).unwrap();
            Ok(Submission { id, form_id, data })
        })
        .unwrap();

    let submissions: Vec<Submission> = submissions_iter.filter_map(|s| s.ok()).collect();
    HttpResponse::Ok().json(submissions)
}
